// -*- C++ -*-
/*!
 * @file  VoiceInFile.cpp
 * @brief Voice input by file component
 * $Date$
 *
 * $Id$
 */

#include "VoiceInFile.h"

// Module specification
// <rtc-template block="module_spec">
static const char* voiceinfile_spec[] =
  {
    "implementation_id", "VoiceInFile",
    "type_name",         "VoiceInFile",
    "description",       "Voice input by file component",
    "version",           "0.1",
    "vendor",            "",
    "category",          "Generic",
    "activity_type",     "COMMUTATIVE",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

VoiceInFile::VoiceInFile(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_FileIn("File", m_File),
    m_SizeOut("Size", m_Size),
    m_WaveOut("Wave", m_Wave),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("File", m_FileIn);
  
  // Set OutPort buffer
  registerOutPort("Size", m_SizeOut);
  registerOutPort("Wave", m_WaveOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

VoiceInFile::~VoiceInFile()
{
}


/*
RTC::ReturnCode_t VoiceInFile::onInitialize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInFile::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInFile::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInFile::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t VoiceInFile::onActivated(RTC::UniqueId ec_id)
{
  // f[^̓|[gɂ݂Ă\̂ŁA
  // f[^̓|[g̓ǂݍ݂sB
  m_FileIn.read();

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t VoiceInFile::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t VoiceInFile::onExecute(RTC::UniqueId ec_id)
{
  const short*  pBuf;
  int           i, iSize;

  // f[^̓|[gɁAVKf[^݂ꍇ
  if( m_FileIn.isNew() ) {

    // f[^̓|[g̃f[^ǂݍ
    m_FileIn.read();

    // f[^̓|[g̃f[^it@CpXjŎw肳ꂽf[^A
    // wavef[^IuWFNgɓǂݍ
    if( m_cWaveData.Load( m_File.data ) ) {

      // f[^TCYi16rbgf[^Zbg̐j擾
      iSize = m_cWaveData.GetCount();
      // f[^TCYAf[^o̓|[gɐݒ
      m_Size.data = iSize;

      // f[^擾
      pBuf = m_cWaveData.GetData();
      fprintf( stdout, "Load %d sets of 16 bits data.\n\n", iSize );
      // f[^Af[^o̓|[gɐݒ
      m_Wave.data.length( iSize );
      for( i = 0; i < iSize; i++ )
        m_Wave.data[i] = pBuf[i];
    }

    // f[^̓Ǎ݂Ɏsꍇ
    else{
      fprintf( stdout, "Error in load data.\n\n" );
      m_Size.data = 0;
      m_Wave.data.length(1);
      m_Wave.data[0] = 0;
    }

    // f[^̃oCgAf[^o̓|[go
    m_WaveOut.write();
    // f[^Af[^o̓|[go
    m_SizeOut.write();
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t VoiceInFile::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInFile::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInFile::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInFile::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInFile::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void VoiceInFileInit(RTC::Manager* manager)
  {
    RTC::Properties profile(voiceinfile_spec);
    manager->registerFactory(profile,
                             RTC::Create<VoiceInFile>,
                             RTC::Delete<VoiceInFile>);
  }
  
};


