/*
  @file   CJuliusRecv.cpp
  @brief  Julius -> adintool ̋NyюMNX
  @author AdIn Research,Inc
  @date   2007/03/03
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>

#include "CJuliusRecv.h"

#define    JR_READ_BUF_SIZE  (1024*64)
#define    JR_TMP_BUF_SIZE   (1024*4)



/*
  @brief  RXgN^
  @author AdIn Research,Inc
  @date   2007/03/03
*/
CJuliusRecv::CJuliusRecv( void )
{
  // ݒl̏
  m_iPortNo = 7654;        // |[gԍ
  strcpy( m_sAdinPath, "/usr/local/bin/adintool" );
  m_iSamplingRate = 48000; // TvO[g
  m_iSelectInterval = 400; // select() ҂
  m_iSegment = 0;          // L 0:A1:L
  m_iRecMax = 5;           // 5b
  m_iSave = 0;             // ^Ȃ

  // ݒt@CAݒlǂݍ
  if( ! loadSetting() ) {
    // ݒt@CǂݎȂꍇ́Aݒlۑ
    saveSetting();
  }

  m_bRunningThread = false;
  m_iJuliusID = m_iListenSock = m_iRecvSock = -1;
  m_bRecv = false;
  m_iRecvSize = m_iRecv = 0;
  m_pRecvBuf = NULL;

  m_sDir[0] = 0x00;
  m_iSeq = 0;
}

/*
  @brief  fXgN^
  @author AdIn Research,Inc
  @date   2007/03/03
*/
CJuliusRecv::~CJuliusRecv( void )
{
  // ̓c[̒~
  if( m_iJuliusID > 0 )
    kill( m_iJuliusID, SIGKILL );

  // listen socket ̊J
  if( m_iListenSock >= 0 )
    close( m_iListenSock );

  // MXbh̒~
  if( m_bRunningThread ) {
    m_bRunningThread = false;
    pthread_join( m_tRecvThread, NULL );
    pthread_detach( m_tRecvThread );
  }

  // obt@̊J
  if( m_pRecvBuf )
    free( (void*)m_pRecvBuf );
}

/*
  @brief  ̓c[N
  @param[in] iPort |[gԍ
  @param[in] sAdinPath  ̓c[̃pX
  @param[in] iSamplingRate TvO[g
  @param[in] iSegmentUse L̐ݒi0:A1:Lj
  @return    int ( < 0Fs, ȊOF )
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
bool CJuliusRecv::Init( void )
{
  // QdR[`FbN
  if( m_bRunningThread ) {
    fprintf( stdout, "CJuliusRecv::Init() called twice.\n" );
    return false;
  }

  // read buffer ̍쐬
  m_iRecvSize = m_iSamplingRate * m_iRecMax * sizeof( short );
  m_iRecv = 0;
  m_bRecv = false;
  m_pRecvBuf = (unsigned char*)malloc( (size_t)m_iRecvSize );

  if( ! m_pRecvBuf ) {
    fprintf( stdout, "Error in malloc(), at CJuliusRecv::Init()\n" );
    return false;
  }

  // ̓c[̐ڑv󂯕t\Pbg쐬
  m_iListenSock = createListen( m_iPortNo );
  if( m_iListenSock < 0 )
    return false;
  
  // ̓c[N
  m_iJuliusID = execJulius( m_iPortNo, m_sAdinPath, m_iSamplingRate, m_iSegment );
  if( m_iJuliusID < 0 )
    return false;
  
  // ̓c[̐ڑv󂯕t
  struct sockaddr_in tClient;
  socklen_t tLen = sizeof( struct sockaddr_in );
  m_iRecvSock = -1;
  while( m_iRecvSock < 0 )
    m_iRecvSock = accept( m_iListenSock, (struct sockaddr*)&tClient, &tLen );

  // ̓c[̃f[^Mp̃Xbh𐶐
  pthread_attr_t tAttr;
  pthread_attr_init( &tAttr );
  m_bRunningThread = false;
  if( pthread_create( &m_tRecvThread, &tAttr, invokeRecvThread, (void*)this ) ) {
    fprintf( stderr, "Error in pthread_create(), errno = %d\n", errno );
    return false;
  }
  while( ! m_bRunningThread );

  return true;
}

/*
  @brief     f[^ۑfBNgݒ肷
  @param[in] sPath ۑfBNgpX
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
void CJuliusRecv::SetDir( const char* sPath )
{
  int  iLen;

  iLen = strlen( sPath );

  if( m_iSave && iLen ) {
    strcpy( m_sDir, sPath );
    if( m_sDir[ iLen - 1 ] == '/' )
      m_sDir[ iLen - 1 ] = 0x00;

    if( ! makeDir( m_sDir ) ) {
      fprintf( stdout, "%s is not exist, and cannot create dir.\n", sPath );
      fprintf( stdout, "Cannot save wave file.\n" );
      m_sDir[0] = 0x00;
    }
  }
}

/*
  @brief     f[^擾
  @return    const char* f[^
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
const short* CJuliusRecv::Read( void )
{
  char sPath[PATH_MAX];

  // ͂ꂽf[^t@Cۑꍇ
  if( m_iSave && strlen( m_sDir ) ) {
    // ۑt@CpX쐬
    sprintf( sPath, "%s/wave%03d.wav", m_sDir, m_iSeq );
    m_iSeq ++;
  }
  else {
    sPath[0] = 0x00;
  }

  // Mf[^TCY
  m_iRecv = 0;

  // f[^MԒtOIɂ
  m_bRecv = true;

  // f[^MԂI܂ő҂
  while( m_bRecv )
    usleep( 1000 * 50 );

  // Mf[^At@Cɕۑ
  if( sPath[0] && m_iRecv ) {
    CWaveData WD;
    WD.Save( sPath, m_iSamplingRate, m_iRecv / sizeof( short ), (short*)m_pRecvBuf );
  }

  return (const short*)m_pRecvBuf;
}

/*
  @brief     f[^TCY擾
  @return    int f[^TCYi16rbgf[^Zbgj
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
int CJuliusRecv::GetSize( void )
{
  return m_iRecv / sizeof( short );
}


//-----------------------------------------------------------------------------
// ݒt@C̐ݒ荀ږ̂`
#define   JR_PORT_NO        "PortNo"
#define   JR_ADIN_PATH      "AdintoolPath"
#define   JR_SAMPLING_RATE  "SamplingRate"
#define   JR_SELECT_INTER   "SelectInterval"
#define   JR_SEGMENT        "SegmentUse"
#define   JR_REC_MAX        "RecordMaxTime"
#define   JR_FILE_OUT       "FileOut"
//-----------------------------------------------------------------------------

/*
  @brief     ݒt@CAݒlǂݍ
  @return    bool
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
bool CJuliusRecv::loadSetting( void )
{
  FILE*	fp;
  char	sBuf[256];
  char* ptr;
  char* ptr1; // ڒl

  // ݒt@CJ
  fp = fopen( JR_INI_NAME, "rt" );
  if( ! fp )
    return false;

  // ݒt@CPsǂݍ
  while( fgets( sBuf, 255, fp ) ) {

    // '='ȍ~̕擾
    ptr = strchr( sBuf, '=' );
    if( ptr ) {

      // '='ȍ~̕ɁA󔒂΁AI[Ƃ
      ptr1 = strchr( ptr, ' ' );
      if( ptr1 ) *ptr1 = 0x00;
      // '='ȍ~̕ɁAs΁AI[Ƃ
      ptr1 = strchr( ptr, '\n' );
      if( ptr1 ) *ptr1 = 0x00;

      // '='̎̈ʒuɈړBptŕAڒl\ƂɂȂB
      ptr++;

      // ږ̕ʂɁAڒlǂݍ
      if( strstr( sBuf, JR_PORT_NO ) )
        m_iPortNo = atoi( ptr );
      else if( strstr( sBuf, JR_ADIN_PATH ) )
        strcpy( m_sAdinPath, ptr );
      else if( strstr( sBuf, JR_SAMPLING_RATE ) )
        m_iSamplingRate = atoi( ptr );
      else if( strstr( sBuf, JR_SELECT_INTER ) )
        m_iSelectInterval = atoi( ptr );
      else if( strstr( sBuf, JR_SEGMENT ) )
        m_iSegment = atoi( ptr );
      else if( strstr( sBuf, JR_REC_MAX ) )
        m_iRecMax = atoi( ptr );
      else if( strstr( sBuf, JR_FILE_OUT ) )
        m_iSave = atoi( ptr );
    }
  }

  // ㏈
  fclose( fp );

  return true;
}


/*
  @brief     ݒt@CɁAݒlۑ
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
void CJuliusRecv::saveSetting( void )
{
  FILE* fp;

  // ݒt@CJ
  fp = fopen( JR_INI_NAME, "wt" );
  if( ! fp ) {
    fprintf( stderr, "Write_file(%s) open error?\n", JR_INI_NAME );
    return;
  }

  // ݒ荀ڒlo
  fprintf( fp, "%s=%d\n", JR_PORT_NO, m_iPortNo );
  fprintf( fp, "%s=%s\n", JR_ADIN_PATH, m_sAdinPath );
  fprintf( fp, "%s=%d\n", JR_SAMPLING_RATE, m_iSamplingRate );
  fprintf( fp, "%s=%d\n", JR_SELECT_INTER, m_iSelectInterval );
  fprintf( fp, "%s=%d\n", JR_SEGMENT, m_iSegment );
  fprintf( fp, "%s=%d\n", JR_REC_MAX, m_iRecMax );
  fprintf( fp, "%s=%d\n", JR_FILE_OUT, m_iSave );

  // ㏈
  fclose( fp );
}




/*
  @brief  ̓c[N
  @param[in] iPort |[gԍ
  @param[in] sAdinPath  ̓c[̃pX
  @param[in] iSamplingRate TvO[g
  @param[in] iSegmentUse ZOggp
  @return int ( < 0Fs, ȊOF )
  @author AdIn Research,Inc
  @date 2007/03/03
*/
int CJuliusRecv::execJulius( int iPort, const char* sAdinPath, int iSamplingRate, int iSegmentUse )
{
  int   iPid;
  char  sCmd[1024];

  // VFR}h쐬
  if( iSegmentUse ) {
    sprintf( sCmd, 
             "%s -in mic -out adinnet -server localhost -port %d -raw -freq %d >& /dev/null",
             sAdinPath, iPort, iSamplingRate );
  }
  else {
    sprintf( sCmd, 
             "%s -in mic -out adinnet -server localhost -port %d -raw -freq %d -nosegment >& /dev/null",
             sAdinPath, iPort, iSamplingRate );
  }

  // vZX𕡐āAqvZX쐬
  iPid = fork();
  if( iPid < 0 ) {
    fprintf( stdout, "Error in fork(), errno = %d\n", errno );
    return iPid;
  }

  // qvZXŁAVFR}hs
  if( iPid == 0 ) {
    system( sCmd );
    exit( -1 );
  }

  return iPid;
}

/*
  @brief     fBNg쐬
  @param[in] sPath fBNgpX
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
bool CJuliusRecv::makeDir( const char* sPath )
{
  struct stat tStat;
  char        sBuf[PATH_MAX];
  char*       ptr;

  // pX̑݃`FbN
  if( stat( sPath, &tStat ) ) {
    // pX݂Ȃꍇ

    if( errno != ENOENT )
      return false;

    // fBNgKw@
    strcpy( sBuf, sPath );
    ptr = strrchr( sBuf, '/' );
    if( ptr ) {
      *ptr = 0x00;
      if( ! makeDir( sBuf ) )
        return false;
    }

    // fBNg쐬
    if( mkdir( sPath, 0755 ) )
      return false;

    return true;
  }

  // pX݂ꍇ
  else {
    // pXfBNg̏ꍇ
    if( S_ISDIR( tStat.st_mode ) )
      return true;

    // pXfBNgȊȌꍇ
    else
      return false;
  }
}


/*
  @brief     ڑv󂯕t|[g쐬
  @param[in] sPath fBNgpX
  @return    int \Pbgʎq
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
int CJuliusRecv::createListen( int iPort )
{
  int   iSock;
  long  lOpt;
  struct sockaddr_in tAddr;

  // \Pbg쐬
  iSock = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );
  if( iSock < 0 ) {
    fprintf( stderr, "Error in socket(), errno = %d\n", errno );
    return iSock;
  }
  // \PbgʐMubLOōs
  fcntl( iSock, F_SETFL, O_NONBLOCK );
  // \PbgIvVݒi|[gɍėpł悤ɂj
  lOpt = 1;
  setsockopt( iSock, 6, SO_REUSEADDR, &lOpt, sizeof( long ) );

  // \Pbg|[gɃoCh
  memset( (void*)&tAddr, 0x00, sizeof( struct sockaddr_in ) );
  tAddr.sin_family = AF_INET;
  tAddr.sin_addr.s_addr = htonl( INADDR_ANY );
  tAddr.sin_port = htons( iPort );
  if( bind( iSock, (struct sockaddr*)&tAddr, sizeof( struct sockaddr_in ) ) ) {
    fprintf( stderr, "Error in bind(), errno = %d\n", errno );
    close( iSock );
    return -1;
  }

  // \Pbgڑv̎t
  listen( iSock, 1 );

  return iSock;
}

/*
  @brief     MXbhN
  @param[in] pParam MNX|C^
  @return    void*
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
void* CJuliusRecv::invokeRecvThread( void* pParam )
{
  return ((CJuliusRecv*)pParam)->onRecv();
}

/*
  @brief     MXbh
  @return    void*
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
void* CJuliusRecv::onRecv()
{
  char sRead[JR_READ_BUF_SIZE];
  int  iRead, iWrite, iMinSize;

  m_bRecv = false;
  m_bRunningThread = true;
  fprintf( stdout, "Ready to recieve.\n" );
  iMinSize = m_iRecvSize / 10;

  while( m_bRunningThread ) {

    // \PbgɃf[^͂܂ő҂
    iRead = readSelect( m_iSelectInterval, m_iRecvSock );
    switch( iRead ) {
    case -1: // G[
      {
        return (void*)0; // XbhI
      }
      break;
    case 0:  // ^CAEg
      if( m_bRecv /* ́Af[^M? */ ) {
        // ŏf[^TCY𖞂ĂH
        if( m_iRecv > iMinSize )
          m_bRecv = false; // f[^MԂI
      }
      break;
    default:
      {
        // \PbgAf[^ǂݍ
        iRead = readBlock( m_iRecvSock, sRead );
        if( iRead < 0 ) { // Ǎ݃G[
          fprintf( stdout, "Error in sock read(), errno = %d\n", errno );
          return (void*)0;
        }

        if( m_bRecv /* ́Af[^M? */ ) {
          if( ! iRead ) {
            // L肠肩Aŏf[^TCY𖞂Ăꍇ
            if( m_iSegment && m_iRecv > iMinSize )
              m_bRecv = false; // f[^MԂI
          }  
          else {
            // Mobt@ɕۑTCY
            if( m_iRecv + iRead > m_iRecvSize )
              iWrite = m_iRecvSize - m_iRecv;
            else
              iWrite = iRead;

            // Mobt@ɕۑ
            memcpy( (void*)( m_pRecvBuf + m_iRecv ), (void*)sRead, iWrite );

            // MTCYXV
            m_iRecv += iWrite;

            // Mobt@tɂȂꍇ
            if( m_iRecv >= m_iRecvSize )
              m_bRecv = false; // f[^MԂI
          }
        }
      }
      break;
    }
  }

  // M\Pbgt`
  close( m_iRecvSock );
  m_iRecvSock = -1;

  return (void*)0;
}

/*
  @brief     \PbgɃf[^͂܂ő҂
  @param[in] iTimeout ^CAEgl
  @param[in] iSock \PbgfBXNv^
  @return    int (-1:G[ / 0:^CAEg / 1:\PbgɃf[^݂
  @author    AdIn Research,Inc
  @date      2007/03/03
*/
int CJuliusRecv::readSelect( int iTimeout, int iSock )
{
  int             iRet;
  fd_set          tRecvFds;
  struct timeval  tTimeOut;

RETRY_PROC:
  // Ώۃ\Pbgw
  FD_ZERO( &tRecvFds );
  FD_SET( iSock, &tRecvFds );

  // ^CAEglw
  tTimeOut.tv_sec  = 0;
  tTimeOut.tv_usec = iTimeout * 1000;

  // \PbgɁAf[^͂̂҂
  iRet = select( FD_SETSIZE, &tRecvFds, 0, 0, &tTimeOut );

  switch( iRet ) {
  //---- G[̏ꍇ -----------------------
  case -1:
    {
      if( errno == EINTR ) {
        goto RETRY_PROC;
      }
      else {
        fprintf( stdout, "Error in select(), errno = %d\n", errno ); 
        return -1;
      }
    }
    break;
  //----- ^CAEg̏ꍇ -----------------
  case 0:
    {
      return 0;
    }
    break;
  //----- \PbgɃf[^͂ꍇ -------
  default:
    {
      // \PbgɃf[^݂H
      if( FD_ISSET( iSock, &tRecvFds ) )
        return 1; // ݂
      else
    	return -1; // ݂Ȃ(G[)
    }
    break;
  }
}

/*
  @brief      \PbgAf[^ǂݍ
  @param[in]  iSock \PbgfBXNv^
  @param[out] sRead ǂݍ񂾃f[^
  	@return   int ( >=0:f[^TCY / -1:G[ )
  @author     AdIn Research,Inc
  @date       2007/03/03
*/
int CJuliusRecv::readBlock( int iSock, char* sRead )
{
  char sWork[JR_TMP_BUF_SIZE];
  int iRet = 0, iRead;
  unsigned short sSize;

  //f[^wb_i4oCgjǂݍ
  while( iRet < 4 ) {

    // \PbgAf[^ǂݍ
    iRet = read( iSock, sWork, JR_TMP_BUF_SIZE );
    if( iRet <= 0 )
      goto READ_ERROR_PROC;

    // f[^wb_̃`FbN - 3oCg, 4oCgڂ̃f[^́A0x00
    if( sWork[2] || sWork[3] )
      iRet = 0; 
  }

  // f[^{̂̃TCY擾if[^wb_̓2oCgɁAf[^TCYi[Ăj
  memcpy( (void*)&sSize, (void*)sWork, sizeof( short ) );

  // Ǎς݃f[^{̂̃TCYvZ
  iRead = iRet - 4;

  // f[^{̂ǂݍ
  if( iRead )

    // ǂݍݍς݃f[^ۑ
    memcpy( (void*)sRead, (void*)( sWork + 4 ), iRead );

    // Ǎ܂ĂȂf[^΁Aǂݍ
    while( iRead < (int)sSize ) {
      // ǂݍ݃TCY
      if( sSize - iRead > JR_TMP_BUF_SIZE )
        iRet = JR_TMP_BUF_SIZE;
      else
        iRet = sSize - iRead;

      // \PbgAf[^ǂݍ
      iRet = read( iSock, sWork, iRet );
      if( iRet < 0 )
        goto READ_ERROR_PROC;
      if( iRet ) {
        // ǂݍݍς݃f[^ۑ
        memcpy( (void*)( sRead + iRead ), (void*)sWork, iRet );
        iRead += iRet;
    }
  }

  return iRead;

// G[
READ_ERROR_PROC:
  fprintf( stdout, "Error in read(), ret = %d, errno = %d\n", iRet, errno );
  return -1;
}
