/*
  @file   CWaveData.cpp
  @brief  wavet@C̃f[^NX(t@C)
  @author AdIn Research,Inc
  @date   2007/03/10
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>
#include "CWaveData.h"

/*---------- WAVE HEADER DATA STRUCTURE ----------*/
typedef struct {
    short          sFormatTag; 
    unsigned short usChannels;
    unsigned int   uiSamplesPerSec;
    unsigned int   uiAvgBytesPerSec;
    unsigned short usBlockAlign; 
    unsigned short usBitsPerSample;
} FMT_CHUNK;
#define FMT_CHUNK_SIZE ( 16 )

typedef struct{
    char           cRIFF[4];
    int            iSizeRIFF;
    char           cType[4];
    char           cFmt[4];
    int            iSizeFmt; 
    FMT_CHUNK      fmtChunk;
    char           cData[4];
    int            iSizeData;
} WAVE_HEADER;
#define WAVE_HEADER_SIZE ( 28 + FMT_CHUNK_SIZE )


/*---------- IMPLEMENT ---------------------------*/

/*
  @brief    RXgN^
  @author   AdIn Research,Inc
  @date     2007/03/10
*/
CWaveData::CWaveData( void )
{
  m_iSize = 0;
  m_pData = NULL;
}

/*
  @brief    fXgN^
  @author   AdIn Research,Inc
  @date     2007/03/10
*/
CWaveData::~CWaveData( void )
{
  clear();
}

/*
  @brief     wavef[^At@Cǂݍ
  @param[in] sPath t@CpX
  @return    bool(true / false)
  @author    AdIn Research,Inc
  @date      2007/03/10
*/
bool CWaveData::Load( const char* sPath )
{
  char     sTmp[64];
  FILE*    fp;
  size_t   tSize;
  struct stat tStat;

  // 
  clear();

  // t@CTCY擾
  if( stat( sPath, &tStat ) ) {
    fprintf( stdout, "Error in stat(%s), errno = %d.\n", sPath, errno );
    return false;
  }

  // wavef[^̗vfi16rbgf[^Zbg̐jvZ
  m_iSize = (int)( ( tStat.st_size - WAVE_HEADER_SIZE ) / sizeof( short ) );

  // wavef[^ǂݍނ߂̗̈m
  m_pData = (short*)malloc( m_iSize * sizeof( short ) );
  if( ! m_pData ) {
    fprintf( stdout, "Error in malloc(%d).\n", m_iSize * sizeof( short ) );
    return false;
  }

  // t@CJ
  fp = fopen( sPath, "rb" );
  if( ! fp ) {
    fprintf( stdout, "Error in fopen(%s), errno = %d.\n", sPath, errno );
    return false;
  }

  // t@Cwb_ǂݍ
  tSize = fread( (void*)sTmp, 1, WAVE_HEADER_SIZE, fp );
  if( tSize != WAVE_HEADER_SIZE ) {
    fprintf( stdout, "Error in fread(), read = %d/%d.\n", tSize, WAVE_HEADER_SIZE );
    fclose( fp );
    return false;
  }

  // wavef[^ǂݍ
  tSize = fread( (void*)m_pData, sizeof( short ), m_iSize, fp );
  if( (int)tSize != m_iSize ) {
    fprintf( stdout, "Error in fread(), read = %d/%d.\n", tSize, m_iSize );
    fclose( fp );
    return false;
  }

  // ㏈
  fclose( fp );

  return true;
}

/*
  @brief     wavef[^At@Cɕۑ
  @param[in] sPath t@CpX
  @param[in] iSamplingRate TvO[g
  @param[in] iSize wavef[^̗vfi16rbgf[^Zbg̐j
  @param[in] pData wavef[^
  @return    bool(true / false)
  @author    AdIn Research,Inc
  @date      2007/03/10
*/
bool CWaveData::Save( const char* sPath, int iSamplingRate, int iSize, const short* pData )
{
  FILE*       fp;
  WAVE_HEADER wh;

  // t@Cwb_쐬
  memcpy( wh.cRIFF, "RIFF", 4 );
  wh.iSizeRIFF = WAVE_HEADER_SIZE + iSize * sizeof(short) - 8;
  memcpy( wh.cType, "WAVE", 4 );
  memcpy( wh.cFmt, "fmt ", 4 );
  wh.iSizeFmt  = FMT_CHUNK_SIZE; 
  wh.fmtChunk.sFormatTag       = 1;
  wh.fmtChunk.usChannels       = 1;
  wh.fmtChunk.uiSamplesPerSec  = iSamplingRate;
  wh.fmtChunk.uiAvgBytesPerSec = iSamplingRate * 1 *  16 / 8 ;
  wh.fmtChunk.usBlockAlign     = 1 * 16 / 8;
  wh.fmtChunk.usBitsPerSample  = 16;
  memcpy( wh.cData, "data", 4 );
  wh.iSizeData = iSize * sizeof(short);

  // t@CJiȂ΁A쐬j
  fp = fopen( sPath, "wb" );
  if( ! fp ) {
    fprintf( stdout, "Error in fopen(%s), errno = %d.\n", sPath, errno );
    return false;
  }

  // wb_o
  fwrite( (void*)&wh.cRIFF,     sizeof(char), 4, fp);
  fwrite( (void*)&wh.iSizeRIFF, sizeof(int),  1, fp);
  fwrite( (void*)&wh.cType,     sizeof(char), 4, fp);
  fwrite( (void*)&wh.cFmt,      sizeof(char), 4, fp);
  fwrite( (void*)&wh.iSizeFmt,  sizeof(int),  1, fp);
  fwrite( (void*)&wh.fmtChunk.sFormatTag, sizeof(short), 1, fp);
  fwrite( (void*)&wh.fmtChunk.usChannels, sizeof(unsigned short), 1, fp);
  fwrite( (void*)&wh.fmtChunk.uiSamplesPerSec, sizeof(unsigned int), 1, fp);
  fwrite( (void*)&wh.fmtChunk.uiAvgBytesPerSec, sizeof(unsigned int),   1, fp);
  fwrite( (void*)&wh.fmtChunk.usBlockAlign, sizeof(unsigned short), 1, fp);
  fwrite( (void*)&wh.fmtChunk.usBitsPerSample, sizeof(unsigned short), 1, fp);
  fwrite( (void*)&wh.cData,     sizeof(char), 4, fp);
  fwrite( (void*)&wh.iSizeData, sizeof(int),  1, fp);

  // wavef[^o
  fwrite( (const void*)pData, sizeof( short ), iSize, fp );

  // ㏈
  fclose( fp );

  return true;
}

/*
  @brief     wavef[^TCY擾
  @return    int wavef[^̗vfi16rbgf[^Zbg̐j
  @author    AdIn Research,Inc
  @date      2007/03/10
*/
int CWaveData::GetCount( void )
{
  return m_iSize;
}

/*
  @brief     wavef[^擾
  @return    const short* wavef[^
  @author    AdIn Research,Inc
  @date      2007/03/10
*/
const short* CWaveData::GetData( void )
{
  return m_pData;
}

/*
  @brief     f[^NX̏
  @author    AdIn Research,Inc
  @date      2007/03/10
*/
void CWaveData::clear( void )
{
  m_iSize = 0;
  if( m_pData ) {
    free( (void*)m_pData );
    m_pData = NULL;
  }
}
