/******************************************************************************
 *      (C)Copyright        AdIn Research, Inc.         1991-1992.            *
 ******************************************************************************
 ******************************************************************************
 *
 *  File name  : rna00def.h
 *
 *  Note       : Beta-RNA Common Library Definitions
 *
 * $Revision: 1.31 $
 * $Author: fujimaki $
 * $Date: 1999/11/09 06:25:12 $
 * $Locker:  $
 *
 *****************************************************************************/
#if !defined(_RNA00DEF_H_)
#define _RNA00DEF_H_

#include "rna00usr.h"
#if defined(MEMDBG)
# include "Kcommon.h"
#endif /* MEMDBG */

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/********** Normalization Table **********/
typedef struct {
    RNUM    rnDim;              /* dimension */
    double *pdReal;             /* input array */
    RANA   *praNorm;            /* output array */
} NORMTBL;

#define NTBL_DIM_MIN            ((RNUM) 2)

/********** System Information Operators (see RSYSINFO) **********/
typedef enum {
    SI_SET,
    SI_ADD,
    SI_SUB
} SYSINFO_OPR;

/********** Sweeping Information **********/
typedef struct {
    RCAT                    rcat;
    Ferr PROTOTYPE((PUBLIC *pfn), (RID));
} RSWEEPINFO;

/********** Matching Type Conflict Detective Structure **********/
typedef struct {
    RMTYPE set;
    RMTYPE conf;
} CONF_TBL;

/********** Matching Mode Predicate **********/
#define FASTMATCH_P(mode, type) \
    ((mode) == RMM_FAST && (((type) & RM_MULMEAN) || ((type) & RM_MIN)))

/********** Error Specification Handler **********/
#if defined(TRAP_ERROR)
# if defined(__STDC__)
#   define rcSetError(ferr, rerr, msg) \
    (fprintf(pErrout_BRNA_, "%s(%d) [%s,%s]\n", \
             __FILE__, __LINE__, #ferr, #rerr), \
     rcSetErrorFn(ferr, rerr, msg))
# else /* not __STDC__ */
#   define rcSetError(ferr, rerr, msg) \
    (fprintf(pErrout_BRNA_, "%s(%d) [ferr,rerr]\n", __FILE__, __LINE__), \
     rcSetErrorFn(ferr, rerr, msg))
# endif /* not __STDC__ */
#else /* not TRAP_ERROR */
# define rcSetError(ferr, rerr, msg)    rcSetErrorFn(ferr, rerr, msg)
#endif /* not TRAP_ERROR */

/********** Fetch Incomplete Functions **********/
#undef NOT_IMPLEMENTED_YET
#define NOT_IMPLEMENTED_YET \
    (fprintf(pErrout_BRNA_,\
             "## Not implemented yet (File:%s, Line:%d)\n",\
             __FILE__, __LINE__),\
     exit(99))

/********** Function Prototypes and Macros **********/

EXPORT Ferr PUBLIC PROTOTYPE(rcSetErrorFn, (Ferr, RERR, const char*));

extern Ferr PROTOTYPE(rcSetSystemInfo, (RCAT, SYSINFO_OPR, RNUM));

extern Ferr PROTOTYPE(rcSweepInfo, (RSWEEPINFO *, int));

extern Ferr PROTOTYPE(rcInitHookEntries, (void));
extern Ferr PROTOTYPE(rcSetHookEntry,
                      (RCAT, RHOOK_ACT, Ferr (*)(GENPTR, RID *, RNUM)));
extern Ferr PROTOTYPE(rcRemoveHookEntry,
                      (RCAT, RHOOK_ACT, Ferr (*)(GENPTR, RID *, RNUM)));
extern Ferr PROTOTYPE(rcCallHook,
                      (RCAT, RHOOK_ACT, GENPTR, RID *, RNUM));

extern Ferr PROTOTYPE(rcIDInitialize, (RCAT));
extern void PROTOTYPE(rcInitIDArray, (RID *, int));
extern Ferr PROTOTYPE(rcIDMaxEntry, (RCAT, int *));
extern Ferr PROTOTYPE(rcIDSetEntry, (RCAT, GENPTR, RID *));
extern Ferr PROTOTYPE(rcIDGetEntry, (RCAT, RID, GENPTR));
extern Ferr PROTOTYPE(rcIDRemoveEntry, (RCAT, RID));
extern RID  PROTOTYPE(rcIDConvToSeq, (RCAT, RID));

extern Ferr PROTOTYPE(rcFuzzyMatch, (RMF *, RANA, RMD *));
extern Ferr PROTOTYPE(rcFuzzyMatchMF, (RMF *, RMF *, RMD *));
extern Ferr PROTOTYPE(rcRawMini, (RMD *, RANA *, RNUM, RNUM, RID, RMD *));
extern Ferr PROTOTYPE(rcWeightMini, (RMD *, RANA *, RNUM, RNUM, RID, RMD *));
extern Ferr PROTOTYPE(rcRawAddMean, (RMD *, RANA *, RNUM, RNUM, RID, RMD *));
extern Ferr PROTOTYPE(rcWeightAddMean, (RMD *, RANA *, RNUM, RNUM, RID, RMD *));
extern Ferr PROTOTYPE(rcRawTimesMean, (RMD *, RANA *, RNUM, RNUM, RID, RMD *));
extern Ferr PROTOTYPE(rcWeightTimesMean, (RMD *, RANA *, RNUM, RNUM, RID, RMD *));
extern Ferr PROTOTYPE(rcRawLinear, (RMD *, RANA *, RNUM, RNUM, RID, RMD *));
extern Ferr PROTOTYPE(rcRuleComp, (RMD *, RANA *, RNUM, RNUM, RID, RMD *));
extern Ferr PROTOTYPE(rcWeightTotalDegree,
                      (RMD, RMTYPE, RANA, RANA *, RNUM, RMD *));
extern Ferr PROTOTYPE(rcValidateMatchType,
                      (RMTYPE, CONF_TBL *, int, RMTYPE *, int, RMTYPE *));
extern Ferr PROTOTYPE(rcSortIDArray, (RID *, GENPTR, int,
                                      int (*)(GENPTR, int, GENPTR, int)));

#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif /* not _RNA00DEF_H_ */
