/******************************************************************************
 *      Copyright      Advanced Intelligence Research, Inc.   1991-1992       *
 ******************************************************************************
 ******************************************************************************
 *
 *  File name  : rnaL0usr.h
 *
 *  Note       : Beta-RNA Learning Module Application Programmer's File
 *
 * $Revision: 1.31 $
 * $Author: fujimaki $
 * $Date: 1999/11/05 03:03:48 $
 * $Locker:  $
 *
 *****************************************************************************/
#if !defined(_RNAL0USR_H_)
#define _RNAL0USR_H_

#include "rna01usr.h"

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/*------------------------------ Initialization Macro -----------------------*/
#if defined(RCINIT_L00)
# undef RCINIT_L00
# define RCINIT_L00     (RLInit00())
#endif /* RCINIT_L00 */

/*------------------------------ Halting Macro ------------------------------*/
#if defined(RCHALT_L00)
# undef RCHALT_L00
# define RCHALT_L00     (RLHalt00())
#endif /* RCHALT_L00 */

/**STDEXT_BEGIN**/
/*----------------------------- Basic Types ---------------------------------*/
typedef double  TSIG;           /* teaching signal (indication) */

/*------------------------ Status & Mode (common) ---------------------------*/
#define MRL_UNSET       (0x00800000L)    /* unset mode */
#define MRL_LEARN       (0x00000001L)    /* learning/none learning */
#define MRL_WEIGHT      (0x00000002L)    /* execute control weight */
#define MRL_NO_CLR_H    (0x00000004L)    /* not clear hist when change MF */
#define MRL_VAREXT      (0x00000008L)    /* variance extend mode */
#define MRL_VARFIX      (0x00000010L)    /* variance fix mode */
#define MRL_ADDVAR      (0x00000020L)    /* avoid zero-varianced MF */
/*------------------------ Status & Mode (pattern set) ----------------------*/
#define MRL_XX1         (0x00000100L)
/*------------------------ Status & Mode (pattern table) --------------------*/
#define MRL_PT_EXT      (0x00001000L)    /* extended mode */
#define MRL_PT_MIN      (0x00002000L)    /* min-data study mode */
#define MRL_PT_COMP     (0x00004000L)    /* PS-PS compensation mode */
/*------------------------ Status & Mode (pattern group) --------------------*/
#define MRL_PG_COMP     (0x00010000L)    /* PT-PT compensation mode */
/*------------------------ Status & Mode (get param) ------------------------*/
#define MRLD_UNDEFINED  (0)
#define MRLD_MF         (1)
#define MRLD_PS         (2)
#define MRLD_PT         (3)
#define MRLD_PG         (4)
/*------------------------ Status & Mode (get matching result) --------------*/
#define RLMM_ACTMF      (0x00000001L)    /* check activity of MF */
#define RLMM_ACTPS      (0x00000002L)    /* check activity of PS */
#define RLMM_ACTPT      (0x00000004L)    /* check activity of PT */
#define RLMM_ACTPG      (0x00000008L)    /* check activity of PG */
#define RLMM_ACT_OFF    (0xfffffff0L)    /* off the bits  for check activity */
#define RLMM_NO_ZERO    (0x00000010L)    /* control MF's MD  0->Small Number */
/*------------------------ Type of Learning (for Info.) ---------------------*/
typedef enum {
    LT_NON = 0, LT_INIT, LT_NORM, LT_EXT, LT_INACT
} LTYPE;

/*----------------------------- MF Learning Parameter -----------------------*/
typedef struct {
    double dHdepth;             /* depth of history buffer */
    double dLvar;               /* gain of left  variance */
    double dRvar;               /* gain of right variance */
    double dLvag;               /* gain of left  vagueness */
    double dRvag;               /* gain of right vagueness */
    double dCutoff;             /* cut-off level */
    double dStdVar;             /* variance  ratio to standard deviation */
    double dStdVag;             /* vagueness ratio to standard deviation */
    RANA   raAddVar;            /* default variance of zero-varianced MF */
} RLMFP;
/*--------------------------- PS Extend Parameter ---------------------------*/
typedef struct {
    RMD  rgThLearn;             /* learning threshold */
    RMD  rgThExtend;            /* extending threshold */
    RANA raDefVar;              /* default variance for extend */
    RANA raDefWeight;           /* default weight   for extend */
} RLEXT;
/*--------------------------- Weight Control Parameter ----------------------*/
typedef struct {
    RMD  rgThMin;               /* threshold (lower) */
    RMD  rgThMax;               /* threshold (upper) */
    RANA raGainDown;            /* gain (dec. weight) */
    RANA raGainUp;              /* gain (inc. weight) */
} RLWEIGHT;
/*--------------------------- PT Conpensation Parameter ---------------------*/
typedef struct {
    double *pdCompPara;         /* compensation parameter */
} RLCOMP;

/*-------------------------------- Rank Table -------------------------------*/
typedef struct {
    RID riPtbl; /* Pattern Tbale ID */
    RID riPset; /* Pattern Set ID */
    RMD rgRes;  /* Mtaching Degree (Sort Key) */ 
    int iIndex; /* Optional Index  (ex. PT order in PG) */
} RRANK;
/**STDEXT_END**/

/*--------------------------- Function Prototype ----------------------------*/
EXPORT Ferr PUBLIC PROTOTYPE(RLInit00, (void));
EXPORT Ferr PUBLIC PROTOTYPE(RLHalt00, (void));
EXPORT Ferr PUBLIC PROTOTYPE(RLSetMFParam, (RID, RID, RID, RLMFP *));
EXPORT Ferr PUBLIC PROTOTYPE(RLSetMFParamPG, (RID, RLMFP *));
EXPORT Ferr PUBLIC PROTOTYPE(RLSetExtParamPT, (RID, RLEXT *));
EXPORT Ferr PUBLIC PROTOTYPE(RLSetExtParamPG, (RID, RLEXT *));
EXPORT Ferr PUBLIC PROTOTYPE(RLSetWeightParamPT, (RID, RLWEIGHT *));
EXPORT Ferr PUBLIC PROTOTYPE(RLSetWeightParamPG, (RID, RLWEIGHT *));
EXPORT Ferr PUBLIC PROTOTYPE(RLSetLearnMode, (RID, RID, RID, unsigned long));
EXPORT Ferr PUBLIC PROTOTYPE(RLSetLearnModePG, (RID, unsigned long));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetMFParam, (RID, RID, RID, RLMFP *, int *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetMFParamPG, (RID, RLMFP *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetExtParamPT, (RID, RLEXT *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetExtParamPG, (RID, RLEXT *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetWeightParamPT, (RID, RLWEIGHT *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetWeightParamPG, (RID, RLWEIGHT *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetLearnMode, (RID, RID, RID, unsigned long *, int *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetLearnModePG, (RID, unsigned long *));
EXPORT Ferr PUBLIC PROTOTYPE(RLActivateData, (RID, RID, RID, RBOOL));
EXPORT Ferr PUBLIC PROTOTYPE(RLActivateDataPG, (RID, RBOOL));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetDataActivity, (RID, RID, RID, RBOOL *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetDataActivityPG, (RID, RBOOL *));
EXPORT Ferr PUBLIC PROTOTYPE(RLInheritData, (RID, RID, RID, RBOOL));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetDataInheritance, (RID, RID, RID, RBOOL *));
EXPORT Ferr PUBLIC PROTOTYPE(RLChangeHistMode, (RID, RID, RID, RBOOL));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetHistMode, (RID, RID, RID, RBOOL *));
EXPORT Ferr PUBLIC PROTOTYPE(RLSetHistInfo, (RID, RID, RID, RNUM, double *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetHistInfo, (RID, RID, RID, RNUM *, double *));
EXPORT Ferr PUBLIC PROTOTYPE(RLMatchEach_Sel, (RID, RID, unsigned long, RBOOL *));
#define RLMatchEach(riPtbl, riSset, ulMode) \
    (RLMatchEach_Sel(riPtbl, riSset, ulMode, (RBOOL *)NULL))
EXPORT Ferr PUBLIC PROTOTYPE(RLMatchEachPG_Sel, (RID, unsigned long, RBOOL *));
#define RLMatchEachPG(riPgrp, ulMode) \
    (RLMatchEachPG_Sel(riPgrp, ulMode, (RBOOL *)NULL))
EXPORT Ferr PUBLIC PROTOTYPE(RLExecLearnPT_Sel, (RID, RID, TSIG, unsigned long,
                                          RLMFP *, RLEXT *, RLWEIGHT *,
                                          RBOOL *));
#define RLExecLearnPT(riPtbl, riSset, tsig, ulStat, prlm, prle, prlw) \
    (RLExecLearnPT_Sel(riPtbl, riSset, tsig, ulStat, prlm, prle, prlw, (RBOOL *)NULL))
EXPORT Ferr PUBLIC PROTOTYPE(RLExecLearnPG_Sel, (RID, RID, TSIG, RBOOL *));
#define RLExecLearnPG(riPgrp, riPtbl, tsig) \
    (RLExecLearnPG_Sel(riPgrp, riPtbl, tsig, (RBOOL *)NULL))
EXPORT Ferr PUBLIC PROTOTYPE(RLGetMatchResult, (RID, RID, unsigned long,
                                         RID *, RMD *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetMatchResultPG, (RID, unsigned long, RMD *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetSortResult, (RID, unsigned long, RNUM, RRANK *));
EXPORT Ferr PUBLIC PROTOTYPE(RLGetSortResultPG, (RID, unsigned long, RNUM, RRANK *));
EXPORT void PUBLIC PROTOTYPE(RLGetLearnInfo, (LTYPE *, RNUM *, RID *, RMD *));
EXPORT Ferr PUBLIC PROTOTYPE(RLCompensateMF, (RMF *, RMF *));

/*--------------------------- Macros (Unset Data) ---------------------------*/
#define RLUnsetMFParam(riPtbl, riPset, riApat) \
    (RLSetMFParam(riPtbl, riPset, riApat, (RLMFP *)NULL))
#define RLUnsetMFParamPG(riPgrp) (RLSetMFParamPG(riPgrp, (RLMFP *)NULL))
#define RLUnsetExtParamPT(riPtbl) (RLSetExtParamPT(riPtbl, (RLEXT *)NULL))
#define RLUnsetExtParamPG(riPgrp) (RLSetExtParamPG(riPgrp, (RLEXT *)NULL))
#define RLUnsetWeightParamPT(riPtbl) \
    (RLSetWeightParamPT(riPtbl, (RLWEIGHT *)NULL))
#define RLUnsetWeightParamPG(riPgrp) \
    (RLSetWeightParamPG(riPgrp, (RLWEIGHT *)NULL))
#define RLUnsetLearnMode(riPtbl, riPset, riApat) \
    (RLSetLearnMode(riPtbl, riPset, riApat, MRL_UNSET))
#define RLUnsetLearnModePG(riPgrp) \
    (RLSetLearnModePG(riPgrp, MRL_UNSET))

/********** Toshi (94/06/09) **********/
typedef struct {
    unsigned long ulNtbl;               /* number of normalization tables */
    unsigned long ulRule;               /* number of matching rules */
    /***** model01 *****/
    unsigned long ulSset;               /* number of sensor sets */
    unsigned long ulPgrp;               /* number of pattern groups */
    unsigned long ulPtbl;               /* number of pattern tables */
    /***** model02 *****/
    unsigned long ulCgrp;               /* number of condition group */
    unsigned long ulCtbl;               /* number of condition table */
    unsigned long ulDtbl;               /* number of data table */
} RSYSSIZE;

EXPORT Ferr PUBLIC PROTOTYPE(RCGetSystemSize, (RSYSSIZE *));

#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif /* not _RNAL01USR_H_ */
